define(["exports", "core/pubsubhub"], function (exports, _pubsubhub) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.biblioDB = exports.name = undefined;

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/biblio-db";

  const ALLOWED_TYPES = new Set(["alias", "reference"]);
  // Database initialization, tracked by "readyPromise"
  const readyPromise = new Promise((resolve, reject) => {
    let request;
    try {
      request = window.indexedDB.open("respec-biblio2", 12);
    } catch (err) {
      return reject(err);
    }
    request.onerror = () => {
      reject(new DOMException(request.error.message, request.error.name));
    };
    request.onsuccess = () => {
      resolve(request.result);
    };
    request.onupgradeneeded = _asyncToGenerator(function* () {
      const db = request.result;
      Array.from(db.objectStoreNames).map(function (storeName) {
        return db.deleteObjectStore(storeName);
      });
      const promisesToCreateSchema = [new Promise(function (resolve, reject) {
        try {
          const store = db.createObjectStore("alias", { keyPath: "id" });
          store.createIndex("aliasOf", "aliasOf", { unique: false });
          store.transaction.oncomplete = resolve;
          store.transaction.onerror = reject;
        } catch (err) {
          reject(err);
        }
      }), new Promise(function (resolve, reject) {
        try {
          const transaction = db.createObjectStore("reference", {
            keyPath: "id"
          }).transaction;
          transaction.oncomplete = resolve;
          transaction.onerror = reject;
        } catch (err) {
          reject(err);
        }
      })];
      try {
        yield Promise.all(promisesToCreateSchema);
        resolve();
      } catch (err) {
        reject(err);
      }
    });
  });

  const biblioDB = exports.biblioDB = {
    get ready() {
      return readyPromise;
    },
    /**
     * Finds either a reference or an alias.
     * If it's an alias, it resolves it.
     *
     * @param {String} id The reference or alias to look for.
     * @return {Object?} The reference or null.
     */
    find(id) {
      var _this = this;

      return _asyncToGenerator(function* () {
        if (yield _this.isAlias(id)) {
          id = yield _this.resolveAlias(id);
        }
        return _this.get("reference", id);
      })();
    },
    /**
     * Checks if the database has an id for a given type.
     *
     * @param {String} type One of the ALLOWED_TYPES.
     * @param {String} id The reference to find.
     * @return {Boolean} True if it has it, false otherwise.
     */
    has(type, id) {
      var _this2 = this;

      return _asyncToGenerator(function* () {
        if (!ALLOWED_TYPES.has(type)) {
          throw new TypeError("Invalid type: " + type);
        }
        if (!id) {
          throw new TypeError("id is required");
        }
        const db = yield _this2.ready;
        return new Promise(function (resolve, reject) {
          var objectStore = db.transaction([type], "readonly").objectStore(type);
          var range = IDBKeyRange.only(id);
          var request = objectStore.openCursor(range);
          request.onsuccess = function () {
            resolve(!!request.result);
          };
          request.onerror = function () {
            reject(new DOMException(request.error.message, request.error.name));
          };
        });
      })();
    },
    /**
     * Checks if a given id is an alias.
     *
     * @param {String} id The reference to check.
     * @return {Promise<Boolean>} Resolves with true if found.
     */
    isAlias(id) {
      var _this3 = this;

      return _asyncToGenerator(function* () {
        if (!id) {
          throw new TypeError("id is required");
        }
        const db = yield _this3.ready;
        return new Promise(function (resolve, reject) {
          var objectStore = db.transaction(["alias"], "readonly").objectStore("alias");
          var range = IDBKeyRange.only(id);
          var request = objectStore.openCursor(range);
          request.onsuccess = function () {
            resolve(!!request.result);
          };
          request.onerror = function () {
            reject(new DOMException(request.error.message, request.error.name));
          };
        });
      })();
    },
    /**
     * Resolves an alias to its corresponding reference id.
     *
     * @param {String} id The id of the alias to look up.
     * @return {Promise<String>} The id of the resolved reference.
     */
    resolveAlias(id) {
      var _this4 = this;

      return _asyncToGenerator(function* () {
        if (!id) {
          throw new TypeError("id is required");
        }
        const db = yield _this4.ready;
        return new Promise(function (resolve, reject) {
          var objectStore = db.transaction("alias", "readonly").objectStore("alias");
          var range = IDBKeyRange.only(id);
          var request = objectStore.openCursor(range);
          request.onsuccess = function () {
            if (request.result === null) {
              return resolve(null);
            }
            resolve(request.result.value.aliasOf);
          };
          request.onerror = function () {
            reject(new DOMException(request.error.message, request.error.name));
          };
        });
      })();
    },
    /**
     * Get a reference or alias out of the database.
     *
     * @param {String} type The type as per ALLOWED_TYPES.
     * @param {[type]} id The id for what to look up.
     * @return {Promise<Object?>} Resolves with the retrieved object, or null.
     */
    get(type, id) {
      var _this5 = this;

      return _asyncToGenerator(function* () {
        if (!ALLOWED_TYPES.has(type)) {
          throw new TypeError("Invalid type: " + type);
        }
        if (!id) {
          throw new TypeError("id is required");
        }
        const db = yield _this5.ready;
        return new Promise(function (resolve, reject) {
          var objectStore = db.transaction([type], "readonly").objectStore(type);
          var range = IDBKeyRange.only(id);
          var request = objectStore.openCursor(range);
          request.onsuccess = function () {
            if (request.result === null) {
              return resolve(null);
            }
            resolve(request.result.value);
          };
          request.onerror = function () {
            reject(new DOMException(request.error.message, request.error.name));
          };
        });
      })();
    },
    /**
     * Adds references and aliases to database. This is usually the data from
     * Specref's output (parsed JSON).
     *
     * @param {Object} data An object that contains references and aliases.
     */
    addAll(data) {
      var _this6 = this;

      return _asyncToGenerator(function* () {
        if (!data) {
          return;
        }
        const aliasesAndRefs = {
          alias: new Set(),
          reference: new Set()
        };
        Object.keys(data).filter(function (key) {
          if (typeof data[key] === "string") {
            let msg = `Legacy SpecRef entries are not supported: \`[[${key}]]\`. `;
            msg += "Please update it to the new format at [specref repo](https://github.com/tobie/specref/)";
            (0, _pubsubhub.pub)("error", msg);
            return false;
          }
          return true;
        }).map(function (id) {
          return Object.assign({ id }, data[id]);
        }).reduce(function (collector, obj) {
          if (obj.aliasOf) {
            collector.alias.add(obj);
          } else {
            collector.reference.add(obj);
          }
          return collector;
        }, aliasesAndRefs);
        const promisesToAdd = Object.keys(aliasesAndRefs).map(function (type) {
          return Array.from(aliasesAndRefs[type]).map(function (details) {
            return _this6.add(type, details);
          });
        }).reduce(function (collector, promises) {
          return collector.concat(promises);
        }, []);
        yield Promise.all(promisesToAdd);
      })();
    },
    /**
     * Adds a reference or alias to the database.
     *
     * @param {String} type The type as per ALLOWED_TYPES.
     * @param {String} details The object to store.
     */
    add(type, details) {
      var _this7 = this;

      return _asyncToGenerator(function* () {
        if (!ALLOWED_TYPES.has(type)) {
          throw new TypeError("Invalid type: " + type);
        }
        if (typeof details !== "object") {
          throw new TypeError("details should be an object");
        }
        if (type === "alias" && !details.hasOwnProperty("aliasOf")) {
          throw new TypeError("Invalid alias object.");
        }
        const db = yield _this7.ready;
        const isInDB = yield _this7.has(type, details.id);
        return new Promise(function (resolve, reject) {
          const store = db.transaction([type], "readwrite").objectStore(type);
          // update or add, depending of already having it in db
          var request = isInDB ? store.put(details) : store.add(details);
          request.onsuccess = resolve;
          request.onerror = function () {
            reject(new DOMException(request.error.message, request.error.name));
          };
        });
      })();
    },
    /**
     * Closes the underlying database.
     *
     * @return {Promise} Resolves after database closes.
     */
    close() {
      var _this8 = this;

      return _asyncToGenerator(function* () {
        const db = yield _this8.ready;
        db.close();
      })();
    }
  };
});
//# sourceMappingURL=biblio-db.js.map