define(["exports", "core/l10n", "core/pubsubhub"], function (exports, _l10n, _pubsubhub) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.run = exports.name = undefined;
  exports.fetch = fetch;
  exports.fetchAll = fetchAll;
  exports.fetchIndex = fetchIndex;

  var _l10n2 = _interopRequireDefault(_l10n);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/github";

  function findNext(header) {
    // Finds the next URL of paginated resources which
    // is available in the Link header. Link headers look like this:
    // Link: <url1>; rel="next", <url2>; rel="foo"; bar="baz"
    // More info here: https://developer.github.com/v3/#link-header
    var m = (header || "").match(/<([^>]+)>\s*;\s*rel="next"/);
    return m && m[1] || null;
  }

  function fetch(url, options) {
    if (options) {
      options.url = url;
      url = options;
    }
    return $.ajax(url);
  }

  function fetchAll(url, options, output = []) {
    var request = fetch(url, options);
    return request.then(function (resp) {
      output.push.apply(output, resp);
      var next = findNext(request.getResponseHeader("Link"));
      return next ? fetchAll(next, options, output) : output;
    });
  }

  function fetchIndex(url, options) {
    // converts URLs of the form:
    // https://api.github.com/repos/user/repo/comments{/number}
    // into:
    // https://api.github.com/repos/user/repo/comments
    // which is what you need if you want to get the index.
    return fetchAll(url.replace(/\{[^}]+\}/, ""), options);
  }

  let run = exports.run = (() => {
    var _ref = _asyncToGenerator(function* (conf) {
      if (!conf.hasOwnProperty("github") || !conf.github) {
        // nothing to do, bail out.
        return;
      }
      if (typeof conf.github === "object" && !conf.github.hasOwnProperty("repoURL")) {
        (0, _pubsubhub.pub)("error", "`respecConf.github` missing property `repoURL`. See https://github.com/w3c/respec/wiki/github");
        return;
      }
      let ghURL;
      try {
        ghURL = new URL(conf.github.repoURL || conf.github);
      } catch (err) {
        (0, _pubsubhub.pub)("error", `\`respecConf.github\` is not a valid URL? (${ghURL})`);
        return;
      }
      if (ghURL.origin !== "https://github.com") {
        (0, _pubsubhub.pub)("error", `\`respecConf.github\` must be HTTPS and pointing to GitHub. (${ghURL})`);
        return;
      }
      const [org, repo] = ghURL.pathname.split("/").filter(function (item) {
        return item;
      });
      if (!org || !repo) {
        (0, _pubsubhub.pub)("error", `\`respecConf.github\` URL needs a path with, for example, w3c/my-spec`);
      }
      const branch = conf.github.branch || "gh-pages";
      const newProps = {
        shortName: repo,
        edDraftURI: `https://${org.toLowerCase()}.github.io/${repo}/`,
        githubAPI: `https://api.github.com/repos/${org}/${repo}`,
        issueBase: `${ghURL.href}${ghURL.pathname.endsWith("/") ? "" : "/"}issues/`
      };
      const commitsHref = `${ghURL.href}${ghURL.pathname.endsWith("/") ? "" : "/"}commits/${branch}`;
      const otherLink = {
        key: conf.l10n.participate,
        data: [{
          value: `GitHub ${org}/${repo}`,
          href: ghURL
        }, {
          value: conf.l10n.file_a_bug,
          href: newProps.issueBase
        }, {
          value: conf.l10n.commit_history,
          href: commitsHref
        }]
      };
      // Write new properties, ignoring existing ones
      Object.getOwnPropertyNames(newProps).filter(function (key) {
        return !conf.hasOwnProperty(key);
      }).map(function (key) {
        return { key, value: newProps[key] };
      }).reduce(function (conf, { key, value }) {
        conf[key] = value;
        return conf;
      }, conf);
      if (!conf.hasOwnProperty("otherLinks")) {
        conf.otherLinks = [];
      }
      conf.otherLinks.unshift(otherLink);
    });

    return function run(_x) {
      return _ref.apply(this, arguments);
    };
  })();
});
//# sourceMappingURL=github.js.map